% For use with Matlab
% Copyright (C) 2020 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Ricardo Maia, Rui Araujo, and  Francisco A.A. Souza. "Self-Evolving Fuzzy Controller Composed of Univariate Fuzzy Control". Applied Sicences, 2020.
% The "uSelf-FLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Ricardo Maia, Rui Araujo, and  Francisco A.A. Souza.
% Self-Evolving Fuzzy Controller Composed of Univariate Fuzzy Control Rules.
% Applied Sciences, 2020, 10(17), 5836.
% DOI: https://doi.org/10.3390/app10175836
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

%% Clean everything
clc
clear all
close all

%% add path
addpath(genpath('./Functions'));

%% configuration file
ParametersConfig % here are the variables that must be definded by the user (Algorithm 1 Steps 1-2)

%% Fuzzy Rules Design Initialization - Algorithm 1 Steps 3-8 
x = Initialization( universe_x, Rules_ini, eta_aux, control_min_max(1) ); % Definition/design of the struct that contain all parameters;

%% Global variables
global safe_interval epsilon Evolve_Parameters x summed_error_window dif_controller delta_dif_controller data_window universe_x

%% Variables Definition
summed_error_window = 0;
data_window = []; % temporal sliding window
dif_controller = [];% the difference of the estimated control error
delta_dif_controller  = [];% the difference of difference of the estimated control error

%% Definition of Evolve_Parameters struct
Evolve_struct_config % here is defined a struct to save some importante parameters to do final plots 

%% Plant (CSTR) initialization
CSTR_initialization

%% Run Simulimk 
RunningSystem = 'uSelfFLC_Simulink';
signalbuilder([RunningSystem '/Reference'], 'set', 'Reference', 'Group 1', time_ref, data_ref);
set_param(RunningSystem,'StartTime','0','StopTime','last_r')
sim(RunningSystem)

%% Final Plots
PlotResults % reference; output; and command signal

Plot_Evolve_Parameters (Evolve_Parameters,delta_dif_controller) % plots of some importante parameters on the evolving stage

PlotMFs(x,10000) % Plots of the final membership functions of all input variables

